#!/usr/bin/perl -w
# $Header: /home/name/projects/tv/RCS/fixmpg2.pl,v 1.4 2012/07/21 00:05:12 name Exp $
#
# fixmpg2.pl
#
# For some unknown reason, recorded .mpg2 file has preceded by 0x18 or 0x70
# bytes of garbage bytes. This interactive program creates a copy of the
# file without the garbage.
# The original .mpg2 file is rename to .mpg2.old.
# The entered offset is checked for correct signature bytes (8 byyes).

use strict;
use File::Basename;
use Fcntl 'SEEK_SET';

my $myname	= basename $0;
my $mydir	= dirname $0;

sub usage {
	my ($ec)	= @_;
	print "fixmpg2 <file> <offset in hex>\n";
	exit 1;
}

our $configfile;
our	$VIDEOROOTDIR;
our	$POSTRECCOMMAND;
our	$CONFIGROOTDIR;
our	$TRANSCODEBITRATE;
our	$AUDIOBITRATE;
require "$mydir/6ga-common.pl";

readConfigFile($configfile);

my $recdir		= "$VIDEOROOTDIR/recorded";

my @recs;

opendir(DIR, $recdir)	|| die "Can't open $recdir\n";
while (my $file	= readdir DIR) {
	if ($file	=~ /^\d{12}-\d{3}-\d{4}-[[:xdigit:]]{8}\.mpg2/) {
		push @recs, $file;
	}
}
closedir DIR;

if ($#recs < 0) {
	print STDERR "** No mpg2 files **\n";
	exit 0;
}

@recs	= sort @recs;
for (my $inx = 0; $inx <= $#recs; $inx++) {
	printf STDERR "%2d: %s\n", $inx, $recs[$inx];
}
print STDERR "Choice? ";
my $choice = <STDIN>;

chomp $choice;
($choice =~ /^\d+$/ && $choice <= $#recs)	|| exit 0;

my $oldfile	= $recs[$choice];
my $newfile = $oldfile;

print STDERR "Offset? ";
my $offset	= <STDIN>;
chomp $offset;

$offset	=~ s/^0[xX]//;
($offset =~ /^[[:xdigit:]]*$/)	|| usage();
my $off	= hex $offset;

(-e "$recdir/$oldfile")	|| die "$oldfile does not exist.\n";

if ($oldfile =~ /\.mpg2$/ && !(-e "$recdir/$oldfile.old")) {
	print STDERR "Renaming $oldfile to $oldfile.old\n";
	rename "$recdir/$oldfile", "$recdir/$oldfile.old"
		|| die "rename $oldfile $oldfile.old failed\n";
	$oldfile	.= ".old";
}
$newfile	=~ s/\.[^\/]*$/.mpg2/;
(-e "$recdir/$newfile")	&& die "$newfile already exists.\n";

open OLD, "$recdir/$oldfile"	|| die "can't open $oldfile\n";
(sysseek(OLD, $off, SEEK_SET) == $off)	|| die "can't seek to 0x$offset\n";
my $buf;
sysread OLD, $buf, 8	|| die "can't read first 8 bytes\n";
unless ($buf eq "\x00\x00\x01\xba\x44\x00\x04\x00") {
	for (my $i = 0; $i < 8; $i++) {
		printf STDERR "%02x ", ord(substr($buf, $i, 1));
	}
	die " - Bad first 8 bytes\n";
}

open NEW, ">$recdir/$newfile"	|| die "can't open $newfile for write\n";
(syswrite(NEW, $buf, 8)	== 8)	|| die "can't write first 8 byte\n";
my $size;
my $count = 0;
print STDERR "$oldfile -> $newfile\n";
while (($size = sysread(OLD, $buf, 1024*1024)) > 0) {
	(syswrite(NEW, $buf, $size) == $size)	|| die "can't write\n";
	printf STDERR "\r%dMB", ++$count;
}
print STDERR "\n";
system "cd $recdir && ls -lrS $oldfile $newfile";
